package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.EXCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.APPEND;
import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_NOT_IN_CONTACT;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import java.util.Optional;

/**
 * The combine creator implementation of {@link BoundaryBiCreaterType#COMBINE} for {@link
 * AbstractGrowingBoundary}. <br> The result is a new boundary with the combined range of both
 * boundaries, or an empty optional, if the creation is not possible. <br> In: {@code b1: [10..20];
 * b2: [15..30]} <br> Out: {@code [10..30]} <br>
 */
public class BiCreaterCombine extends AbstractBoundaryBiCreater<AbstractGrowingBoundary> {

  private static final BiCreaterCombine instance = new BiCreaterCombine();

  private BiCreaterCombine() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static BiCreaterCombine getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return APPEND;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(
      AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    if (b1.checkWith(IS_NOT_IN_CONTACT, b2)) {
      return b1.createBi(APPEND, b2);
    }
    int lC = b1.getLowerBound().compareTo(b2.getLowerBound());
    int uC = b1.getUpperBound().compareTo(b2.getUpperBound());
    return Optional.of(
        b1.getCopy(
            lC < 0 ? b1.getLowerBound() : b2.getLowerBound(),
            uC > 0 ? b1.getUpperBound() : b2.getUpperBound(),
            lC == 0
                ? (b1.getLowerBoundType() == INCLUSIVE || b2.getLowerBoundType() == INCLUSIVE)
                    ? INCLUSIVE
                    : EXCLUSIVE
                : lC < 0 ? b1.getLowerBoundType() : b2.getLowerBoundType(),
            uC == 0
                ? (b1.getUpperBoundType() == INCLUSIVE || b2.getUpperBoundType() == INCLUSIVE)
                    ? INCLUSIVE
                    : EXCLUSIVE
                : lC > 0 ? b1.getUpperBoundType() : b2.getUpperBoundType()));
  }
}
