package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.EXCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.UPPER_BOUNDS;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.IntegerBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.LongBoundary;
import java.util.Optional;

/**
 * The upper bound creator implementation of {@link BoundaryBiCreaterType#UPPER_BOUNDS} for {@link
 * AbstractGrowingBoundary}. <br> The result is a new boundary with the upper bounds, or an empty
 * optional, if the creation is not possible. <br> In: {@code b1: [10..20]; b2: [15..30]} <br> Out:
 * {@code ]20..30]} <br>
 */
public class BiCreaterUpperBounds extends AbstractBoundaryBiCreater<AbstractGrowingBoundary> {

  private static final BiCreaterUpperBounds instance = new BiCreaterUpperBounds();

  private BiCreaterUpperBounds() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static BiCreaterUpperBounds getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return UPPER_BOUNDS;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(
      AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    if (b1 instanceof IntegerBoundary) {
      if (b1.getUpperBound().equals((int) b2.getUpperBound() + 1)
          && b1.getUpperBoundType() == INCLUSIVE
          && b2.getUpperBoundType() == EXCLUSIVE) {
        return Optional.empty();
      }
    }
    if (b1 instanceof LongBoundary) {
      if (b1.getUpperBound().equals((long) b2.getUpperBound() + 1)
          && b1.getUpperBoundType() == INCLUSIVE
          && b2.getUpperBoundType() == EXCLUSIVE) {
        return Optional.empty();
      }
    }
    if (b2.getUpperBound().equals(b2.getMaxValue())) {
      return Optional.empty();
    }
    try {
      return Optional.of(
          b1.getCopy(
              b2.getUpperBound(),
              b1.getUpperBound(),
              b2.getUpperBoundType().getOp(),
              b1.getUpperBoundType()));
    } catch (Exception e) {
      return Optional.empty();
    }
  }
}
