package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;

/**
 * The equal checker implementation of {@link BoundaryCheckType#IS_EQUAL} for {@link
 * StringBoundary}. <br> The check returns true, if the first boundary is equal to the other
 * boundary. <br> E.g: <br>
 *
 * <ul>
 *   <li>b1 = "a"; b2 = "a" -{@literal >} true
 *   <li>b1 = "not("a","b"); b2 = "not("a","b")" -{@literal >} true
 *   <li>b1 = "a","b"; b2 = "a" -{@literal >} false
 * </ul>
 */
public class CheckStringEqual extends AbstractBoundaryCheck<StringBoundary> {

  private static CheckStringEqual instance = new CheckStringEqual();

  private CheckStringEqual() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CheckStringEqual getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_EQUAL;
  }

  @Override
  public boolean check(StringBoundary b1, StringBoundary b2) {
    return checkEqual(b1, b2);
  }

  /** see {@link CheckStringEqual} */
  public static boolean checkEqual(StringBoundary b1, StringBoundary b2) {
    if (b1 == b2 || b1.getText().equals(b2.getText()) || b1.matchesAny() && b2.matchesAny()) {
      return true;
    }
    if (b1.matchesNoneOfValues() != b2.matchesNoneOfValues()
        || b1.getAmountOfElements() != b2.getAmountOfElements()) {
      return false;
    }
    // hashes are sorted while boundary creation
    int[] h1 = b1.getValuesHashes();
    int[] h2 = b2.getValuesHashes();
    for (int i = 0; i < h1.length; i++) {
      if (h1[i] != h2[i]) {
        return false;
      }
    }
    return true;
  }
}
