package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;

/**
 * The not in contact checker implementation of {@link BoundaryCheckType#IS_NOT_IN_CONTACT} for
 * {@link StringBoundary}. <br> The check returns true, if the first boundary is not in contact with
 * the other boundary. <br> E.g: <br>
 *
 * <ul>
 *   <li>b1 = "a"; b2 = "a" -{@literal >} false
 *   <li>b1 = "not("a","b"); b2 = "not("a","b")" -{@literal >} false
 *   <li>b1 = "a","b"; b2 = "c" -{@literal >} true
 * </ul>
 */
public class CheckStringNotInContact extends AbstractBoundaryCheck<StringBoundary> {

  private static CheckStringNotInContact instance = new CheckStringNotInContact();

  private CheckStringNotInContact() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CheckStringNotInContact getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_NOT_IN_CONTACT;
  }

  @Override
  public boolean check(StringBoundary b1, StringBoundary b2) {
    return checkNotInContact(b1, b2);
  }
  /** see {@link CheckStringNotInContact} */
  public static boolean checkNotInContact(StringBoundary b1, StringBoundary b2) {
    return !CheckStringInContact.checkInContact(b1, b2);
  }
}
