package de.unikoblenz.fgbks.base.utils.boundary.creater;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.BoundType;
import java.util.Optional;

/**
 * The upper creator implementation of {@link BoundaryCreaterType#LOWER} for {@link
 * AbstractGrowingBoundary}. <br> The result is the complete range of values, which are lower than
 * the current boundary.<br> In: {@code >5} <br> Out: {@code <=5} <br>
 */
public class CreaterLower extends AbstractBoundaryCreater<AbstractGrowingBoundary> {

  private static final CreaterLower instance = new CreaterLower();

  private CreaterLower() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CreaterLower getInstance() {
    return instance;
  }

  @Override
  public BoundaryCreaterType getType() {
    return BoundaryCreaterType.LOWER;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(AbstractGrowingBoundary b) {
    if (b.getLowerBound().equals(b.getMinValue())) {
      return Optional.empty();
    }
    return Optional.of(
        b.getCopy(
            b.getMinValue(),
            b.getLowerBound(),
            BoundType.INCLUSIVE,
            b.getLowerBoundType().getOp()));
  }
}
