package de.unikoblenz.fgbks.core.dmn.domain.ids;

import de.unikoblenz.fgbks.base.value.AbstractStringValueObject;

/**
 * An AbstractId is a String with a max 100 character length and should specify a unique element of
 * a dmn table.
 */
public abstract class AbstractId extends AbstractStringValueObject {

  /**
   * Create a new copy of the initialValue with the id value of the given Id object.
   *
   * @param initialValue the original id
   */
  public AbstractId(AbstractId initialValue) {
    this(initialValue.getValue());
  }

  /**
   * Create new object with the id value of the initialValue.
   *
   * @param initialValue the value of the new id
   */
  public AbstractId(String initialValue) {
    super(initialValue);
  }

  @Override
  protected Integer getMaxLength() {
    return 100;
  }

  /**
   * Get a string, which contains the classname of the id and the value of the id.
   *
   * @return the String with the classname + ": + the value of the id
   */
  @Override
  public String toString() {
    return this.getClassName() + ": " + this.getValue();
  }
}
