package de.unikoblenz.fgbks.core.dmn.domain.vdmn;

import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionTableId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DefinitionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.RuleId;
import java.util.List;
import java.util.stream.Collectors;
import org.camunda.bpm.model.dmn.HitPolicy;

/**
 * The decision table contains a list of {@link VDmnColumn}s ({@link VDmnInputColumn}s and {@link
 * VDmnOutputColumn}s) and all {@link VDmnRule}s of the table.
 */
public interface VDmnDecisionTable extends VDmnElement {

  /**
   * Get the {@link VDmnDecision} of the decision table.
   *
   * @return the {@link VDmnDecision}
   */
  VDmnDecision getDmnDecision();

  /**
   * Get the {@link HitPolicy} of the decision table.
   *
   * @return the {@link HitPolicy}
   */
  HitPolicy getHitPolicy();

  /**
   * Get all {@link VDmnOutputColumn}s of the decision table.
   *
   * @return a list of {@link VDmnOutputColumn}s
   */
  List<VDmnOutputColumn> getOutputColumns();

  /**
   * Get all {@link VDmnInputColumn}s of the decision table.
   *
   * @return a list of {@link VDmnInputColumn}s
   */
  List<VDmnInputColumn> getInputColumns();

  /**
   * Get all {@link VDmnRule}s of the decision table.
   *
   * @return a list of {@link VDmnRule}s
   */
  List<VDmnRule> getRules();

  /**
   * Get all {@link RuleId}s of the decision table.
   *
   * @return a list of {@link RuleId}s
   */
  default List<RuleId> getRulesIds() {
    return getRules().stream()
        .map(VDmnRule::getRuleId)
        .collect(Collectors.toList()); // TODO optimize (e.g. cache)
  }

  /**
   * Get the {@link DecisionTableId}.
   *
   * @return the {@link DecisionTableId}
   */
  default DecisionTableId getDecisionTableId() {
    return (DecisionTableId) getId();
  }

  /**
   * Get the {@link DefinitionId}.
   *
   * @return the {@link DefinitionId}
   */
  default VDmnDefinition getDmnDefinition() {
    return getDmnDecision().getDmnDefinition();
  }
}
