package de.unikoblenz.fgbks.core.dmn.domain.vdmn;

import de.unikoblenz.fgbks.core.dmn.domain.ids.AbstractId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DefinitionId;
import java.util.List;

/**
 * The definition is the root element for a dmn table and contains all {@link VDmnDecision}s and all
 * {@link VDmnInputData}. Furthermore, all elements of the dmn table can be accessed with {@link
 * VDmnDefinition#getElementById}.
 */
public interface VDmnDefinition extends VDmnElement {

  /**
   * Get the list of {@link VDmnDecision}s.
   *
   * @return the list of {@link VDmnDecision}s
   */
  List<VDmnDecision> getDmnDecisions();

  /**
   * Get the list of {@link VDmnInputData}s.
   *
   * @return the list of {@link VDmnInputData}s
   */
  List<VDmnInputData> getDmnInputData();

  /**
   * Get the vdmn element with the given id.
   *
   * @param id the {@link AbstractId} of the element
   * @return the {@link VDmnElement}, or null, if the element was not found
   */
  VDmnElement getElementById(AbstractId id);

  /**
   * Get the {@link DefinitionId}.
   *
   * @return the {@link DefinitionId}
   */
  default DefinitionId getDefinitionId() {
    return (DefinitionId) getId();
  }
}
