package de.unikoblenz.fgbks.core.dmn.domain.vdmn;

import de.unikoblenz.fgbks.base.domain.Name;
import java.util.List;
import java.util.Optional;

/**
 * This interface represents a node on DRD level. A node can either be a {@link VDmnDecision} or a
 * {@link VDmnInputData} node.
 */
public interface VDmnNode extends VDmnElement {

  /**
   * Get the {@link VDmnDefinition}.
   *
   * @return the {@link VDmnDefinition}
   */
  VDmnDefinition getDmnDefinition();

  /**
   * Get a list of {@link VDmnDecision}s, which has this current node as information requirement.
   * See also {@link VDmnDecision#getDmnInformationRequirements()}.
   *
   * @return a list of {@link VDmnDecision}s
   */
  List<VDmnDecision> getInformationProvidingDecisions();

  /**
   * Get the name of the node, if present.
   *
   * @return an optional {@link Name}
   */
  Optional<Name> getName();

  /**
   * Check, if the node is a {@link VDmnDecision} node.
   *
   * @return true, if the node is a {@link VDmnDecision}
   */
  default boolean isDecision() {
    return this instanceof VDmnDecision;
  }

  /**
   * Check, if the node is a {@link VDmnInputData} node.
   *
   * @return true, if the node is a {@link VDmnInputData}
   */
  default boolean isInputData() {
    return this instanceof VDmnInputData;
  }
}
