package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDefinition;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnNode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.apache.commons.lang3.Validate;

/**
 * Implementation of {@link VDmnDecision}
 */
public class VDmnDecisionImpl extends AbstractVDmnNode implements VDmnDecision {

  private VDmnDecisionTable dmnDecisionTable;
  private List<VDmnNode> informationRequirements;

  private VDmnDecisionImpl() {
    super();
    informationRequirements = new ArrayList<>();
  }

  @Override
  public List<VDmnNode> getDmnInformationRequirements() {
    return Collections.unmodifiableList(informationRequirements);
  }

  @Override
  public VDmnDecisionTable getDmnDecisionTable() {
    return dmnDecisionTable;
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnDecisionImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnDecisionImpl> {

    public Builder withDmnDefinition(VDmnDefinition dmnDefinition) {
      value.dmnDefinition = dmnDefinition;
      return this;
    }

    public Builder withDecisionId(DecisionId decisionId) {
      value.id = decisionId;
      return this;
    }

    public Builder withDmnDecisionTable(VDmnDecisionTable dmnDecisionTable) {
      value.dmnDecisionTable = dmnDecisionTable;
      return this;
    }

    public Builder withDmnDecisionTableFromBuilder(
        VDmnDecisionTableImpl.Builder dmnDecisionTableBuilder) {
      withDmnDecisionTable(dmnDecisionTableBuilder.getUnbuildValue());
      dmnDecisionTableBuilder.withDmnDecision(value);
      return this;
    }

    public Builder addInformationRequirements(VDmnNode vDmnNode) {
      value.informationRequirements.add(Validate.notNull(vDmnNode));
      return this;
    }

    public Builder addInformationRequirementsFromBuilder(
        VDmnDecisionImpl.Builder vDmnDecisionBuilder) {
      addInformationRequirements(vDmnDecisionBuilder.getUnbuildValue());
      vDmnDecisionBuilder.addInformationProvidingDecisions(value);
      return this;
    }

    public Builder addInformationRequirementsFromBuilder(
        VDmnInputDataImpl.Builder vDmnInputDataBuilder) {
      addInformationRequirements(vDmnInputDataBuilder.getUnbuildValue());
      vDmnInputDataBuilder.addInformationProvidingDecisions(value);
      return this;
    }

    public Builder addInformationProvidingDecisions(VDmnDecision dmnDecision) {
      value.informationProvidingDecisions.add(Validate.notNull(dmnDecision));
      return this;
    }

    public Builder addInformationProvidingDecisionsFromBuilder(
        VDmnDecisionImpl.Builder vDmnDecisionBuilder) {
      addInformationProvidingDecisions(vDmnDecisionBuilder.getUnbuildValue());
      vDmnDecisionBuilder.addInformationRequirements(value);
      return this;
    }

    public Builder withName(String name) {
      value.name = new Name(name);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.dmnDefinition);
      Validate.notNull(value.id);
      Validate.notNull(value.dmnDecisionTable);
    }
  }
}
