package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputDataId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDefinition;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputData;
import org.apache.commons.lang3.Validate;

/**
 * Implementation of {@link VDmnInputData}.
 */
public class VDmnInputDataImpl extends AbstractVDmnNode implements VDmnInputData {

  private VDmnInputDataImpl() {
    super();
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnInputDataImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnInputDataImpl> {

    public Builder withDmnDefinition(VDmnDefinition dmnDefinition) {
      value.dmnDefinition = dmnDefinition;
      return this;
    }

    public Builder withInputDataId(InputDataId inputDataId) {
      value.id = inputDataId;
      return this;
    }

    public Builder addInformationProvidingDecisions(VDmnDecision dmnDecision) {
      value.informationProvidingDecisions.add(Validate.notNull(dmnDecision));
      return this;
    }

    public Builder addInformationProvidingDecisionsFromBuilder(
        VDmnDecisionImpl.Builder vDmnDecisionBuilder) {
      addInformationProvidingDecisions(vDmnDecisionBuilder.getUnbuildValue());
      vDmnDecisionBuilder.addInformationRequirements(value);
      return this;
    }

    public Builder withName(String name) {
      value.name = new Name(name);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.dmnDefinition);
      Validate.notNull(value.id);
    }
  }
}
