package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.base.domain.Label;
import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.base.domain.StringValue;
import de.unikoblenz.fgbks.core.dmn.domain.ids.OutputId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import org.apache.commons.lang3.Validate;

/**
 * Implementation of {@link VDmnOutputColumn}.
 */
public class VDmnOutputColumnImpl extends AbstractVDmnColumnImpl implements VDmnOutputColumn {

  private VDmnOutputColumnImpl() {
    super();
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnOutputColumnImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnOutputColumnImpl> {

    public Builder withOutputId(OutputId outputId) {
      value.id = outputId;
      return this;
    }

    public Builder withDecisionTable(VDmnDecisionTable decisionTable) {
      value.dmnDecisionTable = decisionTable;
      return this;
    }

    public Builder withTypeRef(VTypeRef typeRef) {
      value.typeRef = typeRef;
      return this;
    }

    public Builder addValue(VDmnOutputValue dmnValue) {
      value.values.add(Validate.notNull(dmnValue));
      return this;
    }

    public Builder withName(String name) {
      value.name = new Name(name);
      return this;
    }

    public Builder withLabel(String label) {
      value.label = new Label(label);
      return this;
    }

    public Builder addValueFromBuilder(VDmnOutputValueImpl.Builder dmnValueBuilder) {
      addValue(Validate.notNull(dmnValueBuilder.getUnbuildValue()));
      dmnValueBuilder.withDmnOutputColumn(value);
      return this;
    }

    public Builder addPredefinedInputValue(String predefinedValue) {
      value.predefinedValues.add(new StringValue(predefinedValue));
      return this;
    }

    public Builder addPredefinedInputValues(String[] predefinedValues) {
      Validate.noNullElements(predefinedValues);
      for (String s : predefinedValues) {
        addPredefinedInputValue(s);
      }
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.typeRef);
      Validate.notNull(value.dmnDecisionTable);
    }
  }
}
