package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.base.domain.Description;
import de.unikoblenz.fgbks.base.domain.Text;
import de.unikoblenz.fgbks.core.dmn.domain.ids.OutputEntryId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import org.apache.commons.lang3.Validate;

/**
 * Implementation of {@link VDmnOutputValue}.
 */
public class VDmnOutputValueImpl extends AbstractVDmnValue implements VDmnOutputValue {

  private VDmnOutputValueImpl() {
    super();
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnOutputValueImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnOutputValueImpl> {

    public Builder withOutputEntryId(OutputEntryId outputEntryId) {
      value.id = outputEntryId;
      return this;
    }

    public Builder withDmnRule(VDmnRule dmnRule) {
      value.dmnRule = dmnRule;
      return this;
    }

    public Builder withText(String text) {
      value.text = new Text(text);
      return this;
    }

    public Builder withDescription(String description) {
      value.description = new Description(description);
      return this;
    }

    public Builder withDmnOutputColumn(VDmnOutputColumn dmnColumn) {
      value.dmnColumn = dmnColumn;
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.dmnRule);
      Validate.notNull(value.text);
      Validate.notNull(value.dmnColumn);
    }
  }
}
