package de.unikoblenz.fgbks.core.dmn.utils;

import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDefinition;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.parser.SimpleVDmnParser;
import java.nio.charset.Charset;
import java.util.List;
import java.util.Optional;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import org.apache.commons.io.IOUtils;
import org.camunda.bpm.dmn.engine.DmnDecision;
import org.camunda.bpm.dmn.engine.DmnEngineConfiguration;
import org.camunda.bpm.model.dmn.DmnModelInstance;
import org.camunda.bpm.model.dmn.impl.DmnParser;

/**
 * Application scoped service, which creates a {@link DmnObjectContainer} with the method {@link
 * DmnService#getDmnObjectContainerFromXml(String)}. This container contained the different model
 * types of a dmn table.
 */
@ApplicationScoped
public class DmnService {

  @Inject SimpleVDmnParser simpleDmnParser;

  /**
   * Get an optional of {@link DmnObjectContainer} from the given xml string of a dmn table. The
   * optional is empty, if the dmn xml can not be parsed.
   *
   * @param dmnXmlString the xml of a dmn table
   * @return a optional of a new {@link DmnObjectContainer}
   */
  public Optional<DmnObjectContainer> getDmnObjectContainerFromXml(String dmnXmlString) {
    try {
      List<DmnDecision> dmnDecisions =
          DmnEngineConfiguration.createDefaultDmnEngineConfiguration()
              .buildEngine()
              .parseDecisions(IOUtils.toInputStream(dmnXmlString, Charset.defaultCharset()));
      DmnModelInstance dmnModel =
          new DmnParser()
              .parseModelFromStream(IOUtils.toInputStream(dmnXmlString, Charset.defaultCharset()));
      VDmnDefinition vDmnDefinition = simpleDmnParser.getVDmnDefinition(dmnModel);
      return Optional.of(new DmnObjectContainer(dmnDecisions, dmnModel, vDmnDefinition));
    } catch (Exception e) {
      return Optional.empty();
    }
  }
}
