package de.unikoblenz.fgbks.core.dmn.verification.metrics;

import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.VerificationType;
import java.util.HashMap;
import java.util.Map;
import javax.enterprise.context.ApplicationScoped;

/**
 * Service class for adding statistics of execution time of {@link VerificationType}s with the
 * function {@link DmnVerificationMetricsService#addExecutionTime}. The statistic metrics can be
 * accessed with {@link DmnVerificationMetricsService#getMetrics(String)} to get a {@link MetricSet}
 * object.
 */
@ApplicationScoped
public class DmnVerificationMetricsService {

  public static final String PUBLIC_TOKEN = "all";
  private MetricSet publicMetrics;
  private Map<String, MetricSet> metricSets;

  protected DmnVerificationMetricsService() {
    metricSets = new HashMap<>();
    publicMetrics = new MetricSet();
    metricSets.put(PUBLIC_TOKEN, publicMetrics);
  }

  /**
   * Get the current {@link Metric}s in a {@link MetricSet}.
   *
   * @param token the token for metrics
   * @return the {@link MetricSet}
   */
  public MetricSet getMetrics(String token) {
    return metricSets.getOrDefault(token, new MetricSet());
  }

  /**
   * Add a execution time tho the given {@link VerificationType}.
   *
   * @param token the token for different users
   * @param type the {@link VerificationType}
   * @param timeInNs the time in ns
   */
  public void addExecutionTime(
      String token, VerificationType type, long timeInNs, long numberOfElements) {
    if (!token.equals(PUBLIC_TOKEN)) {
      MetricSet metricSet = metricSets.getOrDefault(token, new MetricSet());
      metricSets.put(token, metricSet);
      metricSet.addExecutionTime(type, timeInNs, numberOfElements);
    }
    publicMetrics.addExecutionTime(type, timeInNs, numberOfElements);
  }
}
