package de.unikoblenz.fgbks.core.dmn.verification.metrics;

import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.VerificationType;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import javax.json.bind.annotation.JsonbProperty;
import org.apache.commons.lang3.Validate;

/**
 * A metric set contains all metrics.
 */
public class MetricSet implements Serializable {

  private Set<Metric> metrics = new HashSet<>();

  /**
   * Get all metrics.
   *
   * @return the set of {@link Metric}s
   */
  @JsonbProperty("verificationMetrics")
  public Set<Metric> getMetrics() {
    return new HashSet<>(metrics);
  }

  /**
   * Add a new execution time of a verifier to the metric map
   *
   * @param type the {@link VerificationType}
   * @param timeInNs the time in ns
   * @param numberOfElements the number of elements
   */
  public synchronized void addExecutionTime(
      VerificationType type, long timeInNs, long numberOfElements) {
    Validate.notNull(type);
    Metric m =
        metrics.stream().filter(x -> x.type.equals(type)).findFirst().orElse(new Metric(type));
    m.addExecution(timeInNs, numberOfElements);
    metrics.add(m);
  }
}
