package de.unikoblenz.fgbks.core.dmn.verification.result.actions;

import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.DECISION;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.INPUT_COLUMN;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.INPUT_DATA;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.INPUT_ENTRY;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.OUTPUT_COLUMN;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.OUTPUT_ENTRY;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.RULE;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionType.SHOW;

import de.unikoblenz.fgbks.base.builder.DefaultBuilder;
import de.unikoblenz.fgbks.core.dmn.verification.result.AbstractResultObject;
import java.util.HashMap;
import javax.json.bind.annotation.JsonbProperty;
import org.apache.commons.lang3.Validate;

public class Action extends AbstractResultObject {

  public static Action ACTION_SHOW_INPUT_DATA;
  public static Action ACTION_SHOW_DECISION;
  public static Action ACTION_SHOW_RULES;
  public static Action ACTION_SHOW_INPUT_ENTRIES;
  public static Action ACTION_SHOW_OUTPUT_ENTRIES;
  public static Action ACTION_SHOW_INPUT_COLUMNS;
  public static Action ACTION_SHOW_OUTPUT_COLUMNS;

  static {
    ACTION_SHOW_INPUT_DATA = getBuilder().withActionType(SHOW).withActionScope(INPUT_DATA).build();
    ACTION_SHOW_DECISION = getBuilder().withActionType(SHOW).withActionScope(DECISION).build();
    ACTION_SHOW_RULES = getBuilder().withActionType(SHOW).withActionScope(RULE).build();
    ACTION_SHOW_INPUT_ENTRIES =
        getBuilder().withActionType(SHOW).withActionScope(INPUT_ENTRY).build();
    ACTION_SHOW_OUTPUT_ENTRIES =
        getBuilder().withActionType(SHOW).withActionScope(OUTPUT_ENTRY).build();
    ACTION_SHOW_INPUT_COLUMNS =
        getBuilder().withActionType(SHOW).withActionScope(INPUT_COLUMN).build();
    ACTION_SHOW_OUTPUT_COLUMNS =
        getBuilder().withActionType(SHOW).withActionScope(OUTPUT_COLUMN).build();
  }

  private ActionType actionType;
  private ActionScope actionScope;
  private HashMap<String, String> values;

  private Action() {
    values = new HashMap<>();
  }

  @JsonbProperty("actionType")
  public ActionType getActionType() {
    return actionType;
  }

  @JsonbProperty("actionScope")
  public ActionScope getActionScope() {
    return actionScope;
  }

  @JsonbProperty("actionValues")
  public HashMap<String, String> getValue() {
    return new HashMap<>(values);
  }

  public static Builder getBuilder() {
    return new Action().new Builder();
  }

  public class Builder extends DefaultBuilder<Action> {

    public Builder withActionType(ActionType actionType) {
      value.actionType = actionType;
      return this;
    }

    public Builder withActionScope(ActionScope actionScope) {
      value.actionScope = actionScope;
      return this;
    }

    public Builder addValue(String key, String value) {
      this.value.values.put(Validate.notNull(key), Validate.notNull(value));
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.actionScope);
      Validate.notNull(value.actionType);
    }
  }
}
