package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.impl.DateBoundary.dateGroupName;
import static de.unikoblenz.fgbks.base.utils.boundary.impl.DateBoundary.datePattern;
import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.templateDecisionRule;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_INPUT_ENTRIES;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_OUTPUT_ENTRIES;

import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.DateVerification;
import java.time.LocalDateTime;
import java.time.format.DateTimeParseException;
import java.util.List;
import java.util.regex.Matcher;

@DmnVerifier(verifierType = DateVerification.class)
public class DateVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    List<VDmnColumn> dateColumns =
        VDmnFunctions.getColumnsByTypeRef(dmnObjectContainer.getVDmnDefinition(), VTypeRef.DATE);
    for (VDmnColumn dateColumn : dateColumns) {
      checkDateColumn(dateColumn);
    }
  }

  private void checkDateColumn(VDmnColumn dateColumn) {
    for (VDmnValue dateValue : dateColumn.getValues()) {
      checkDateValue(dateValue);
    }
  }

  private void checkDateValue(VDmnValue dateValue) {
    if (!dateValue.getText().getValue().isEmpty()) {
      String[] vals =
          dateValue
              .getText()
              .getValue()
              .replace('<', '\0')
              .replace('>', '\0')
              .replace('[', '\0')
              .replace(']', '\0')
              .trim()
              .split("\\.\\.");
      for (String dateStringValue : vals) {
        checkCorrectDate(dateValue, dateStringValue);
      }
    }
  }

  private void checkCorrectDate(VDmnValue dateValue, String dateStringValue) {
    // 1. check pattern
    Matcher matcher = datePattern.matcher(dateStringValue);
    if (!matcher.find()) {
      vreFactory.addElement(VerificationResultEntryElement.create(dateValue));
      vreFactory.addToEntry(
          VerificationClassification.FATAL_ERROR,
          templateDecisionRule(dateValue.getDmnRule())
              + "Date value '%s' does not match 'date and time(\"yyyy-mm-ddTHH:MM:SS\").'",
          dateStringValue);
    } else {
      // 2. check correct date
      String dateTime = matcher.group(dateGroupName);
      try {
        LocalDateTime.parse(dateTime);
      } catch (DateTimeParseException e) {
        vreFactory.addElement(VerificationResultEntryElement.create(dateValue));
        vreFactory.addVerificationFix(
            dateValue.isInputValue() ? SHOW_INPUT_ENTRIES : SHOW_OUTPUT_ENTRIES);
        vreFactory.addToEntry(
            VerificationClassification.FATAL_ERROR,
            templateDecisionRule(dateValue.getDmnRule())
                + "Date value '%s' has no valid date format.",
            dateTime);
      }
    }
  }
}
