package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.templateDecisionColumn;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_INPUT_ENTRIES;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_OUTPUT_ENTRIES;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import de.unikoblenz.fgbks.base.wordnet.WordnetService;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.EquivalentStringVerification;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@DmnVerifier(verifierType = EquivalentStringVerification.class)
public class EquivalentVerifier extends AbstractVerifier {

  WordnetService wordnetService;

  @Override
  protected void doVerification() {
    wordnetService = WordnetService.getInstance();
    VDmnFunctions.getColumnsByTypeRef(dmnObjectContainer.getVDmnDefinition(), VTypeRef.STRING)
        .forEach(this::checkStringColumnForSynonyms);
  }

  private void checkStringColumnForSynonyms(VDmnColumn c) {
    List<String> stringVals = new ArrayList<>();
    List<String> stringValsInner = new ArrayList<>();
    List<VDmnValue> values = c.getValues();
    for (int i = 0; i < values.size(); i++) {
      VDmnValue v1 = values.get(i);
      if (!v1.getText().getValue().isEmpty()) {
        stringVals.clear();
        if (v1.isOutputValue()) {
          stringVals.add(v1.getText().getValue().replace('"', '\0').trim());
        } else {
          stringVals.addAll(
              Arrays.asList(((StringBoundary) ((VDmnInputValue) v1).getBoundary()).getValues()));
        }
        for (int u = i + 1; u < values.size(); u++) {
          VDmnValue v2 = values.get(u);
          if (!v2.getText().getValue().isEmpty()) {
            stringValsInner.clear();
            if (v2.isOutputValue()) {
              stringValsInner.add(v2.getText().getValue().replace('"', '\0').trim());
            } else {
              stringValsInner.addAll(
                  Arrays.asList(
                      ((StringBoundary) ((VDmnInputValue) v2).getBoundary()).getValues()));
            }
            compareStrings(v1, v2, stringVals, stringValsInner);
          }
        }
      }
    }
  }

  private void compareStrings(
      VDmnValue v1, VDmnValue v2, List<String> stringVals, List<String> stringValsInner) {
    for (int i = 0; i < stringVals.size(); i++) {
      for (int u = 0; u < stringValsInner.size(); u++) {
        if (wordnetService.areNounsSynonyms(stringVals.get(i), stringValsInner.get(u))) {
          vreFactory.addElement(VerificationResultEntryElement.create(v1));
          vreFactory.addElement(VerificationResultEntryElement.create(v2));
          vreFactory.addVerificationFix(
              v1.isInputValue() ? SHOW_INPUT_ENTRIES : SHOW_OUTPUT_ENTRIES);
          vreFactory.addToEntry(
              VerificationClassification.WARNING,
              templateDecisionColumn(v1.getDmnColumn())
                  + "String values \"%s\" (Row %s) and \"%s\" (Row %s) might be equivalent.",
              stringVals.get(i),
              v1.getDmnRule().getRowNumber().toString(),
              stringValsInner.get(u),
              v2.getDmnRule().getRowNumber().toString());
        }
      }
    }
  }
}
