package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_DECISION;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_INPUT_DATA;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.DEFAULT_SHOW_NAME;

import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputData;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnNode;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.MissingInputColumnVerification;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@DmnVerifier(verifierType = MissingInputColumnVerification.class)
public class MissingInputColumnVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    // 1. check for input data nodes
    for (VDmnInputData inputData : dmnObjectContainer.getVDmnDefinition().getDmnInputData()) {
      if (inputData.getName().isPresent()) {
        List<Name> inputDataNames = Collections.singletonList(inputData.getName().get());
        checkInputData(inputData, inputDataNames);
      }
    }
    // 2. check for decision nodes
    for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getDmnDecisions()) {
      List<Name> inputDataNames =
          decision.getDmnDecisionTable().getOutputColumns().stream()
              .map(c -> c.getName().orElse(null))
              .filter(Objects::nonNull)
              .collect(Collectors.toList());
      checkInputData(decision, inputDataNames);
    }
  }

  private void checkInputData(VDmnNode inputDataNode, List<Name> inputDataNames) {
    // Get all outgoing decisions from the current input node
    outer:
    for (VDmnDecision decision : inputDataNode.getInformationProvidingDecisions()) {
      // Check all input columns of the decision node
      List<Name> inputColumnsNames =
          decision.getDmnDecisionTable().getInputColumns().stream()
              .map(x -> x.getName().orElse(null))
              .filter(Objects::nonNull)
              .collect(Collectors.toList());
      for (Name inDataName : inputDataNames) {
        if (inputColumnsNames.contains(inDataName)) {
          continue outer;
        }
      }
      vreFactory
          .addElement(VerificationResultEntryElement.create(inputDataNode))
          .addElement(VerificationResultEntryElement.create(decision))
          .addVerificationFix(
              VerificationFix.getBuilder()
                  .withFixName(DEFAULT_SHOW_NAME)
                  .addAction(ACTION_SHOW_INPUT_DATA)
                  .addAction(ACTION_SHOW_DECISION)
                  .build())
          .addToEntry(
              VerificationClassification.WARNING,
              (inputDataNode.isInputData() ? "Input data" : "Decision")
                  + " node \"%s\" has no corresponding input column in decision \"%s\"",
              inputDataNode.getName().orElse(Name.NO_NAME),
              decision.getName().orElse(Name.NO_NAME));
    }
  }
}
