package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.getColumnStringName;
import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.templateDecision;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_INPUT_COLUMNS;

import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnNode;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.MissingInputDataVerification;
import java.util.List;

@DmnVerifier(verifierType = MissingInputDataVerification.class)
public class MissingInputDataVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getDmnDecisions()) {
      List<VDmnNode> inputNodes = decision.getDmnInformationRequirements();
      // Check for each input column if there is one input data node or one output column from a
      // decision node
      for (VDmnInputColumn inputColumn : decision.getDmnDecisionTable().getInputColumns()) {
        checkExistingInputData(inputColumn, inputNodes);
      }
    }
  }

  private void checkExistingInputData(VDmnInputColumn inputColumn, List<VDmnNode> inputNodes) {
    // if the inputColumn has no name, add the column
    if (inputColumn.getName().isPresent()) {
      Name inputColumnName = inputColumn.getName().get();
      // check, if there is one input Data with the same name
      for (VDmnNode node : inputNodes) {
        // input Data node, check only the name
        if (node.isInputData()) {
          if (node.getName().isPresent() && node.getName().get().equals(inputColumnName)) {
            return;
          }
        } else if (node.isDecision()) {
          VDmnDecision decision = (VDmnDecision) node;
          // check for each output column, if the name of the output column matches the input column
          // name
          for (VDmnOutputColumn outputColumn : decision.getDmnDecisionTable().getOutputColumns()) {
            if (outputColumn.getName().isPresent()
                && outputColumn.getName().get().equals(inputColumnName)) {
              return;
            }
          }
        }
      }
    }
    // no input data found yet -> add column to results
    vreFactory
        .addElement(VerificationResultEntryElement.create(inputColumn))
        .addVerificationFix(SHOW_INPUT_COLUMNS)
        .addToEntry(
            VerificationClassification.WARNING,
            templateDecision(inputColumn.getDmnDecision())
                + "Input column \"%s\" has no input data node.",
            getColumnStringName(inputColumn));
  }
}
