package de.unikoblenz.fgbks.core.dmn.verification.verifier.types;

import de.unikoblenz.fgbks.base.domain.Description;
import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.classification.ClassificationType;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.classification.DecisionLogicLevelVerification;
import javax.validation.constraints.NotNull;

@Type
public class EquivalentStringVerification extends AbstractVerificationType {

  private static final EquivalentStringVerification instance = new EquivalentStringVerification();

  private EquivalentStringVerification() {
    super(
        new Name("EquivalentStringVerification"),
        new Name("Equivalent Strings"),
        new Description(
            "Detecting rules which are not identical, "
                + "but still semantically equivalent. "
                + "Here, the verifier checks if there exist multiple rules "
                + "which use synonyms as inputs and are therefore equivalent, "
                + "based on synonym relations via Wordnet."));
  }

  public static EquivalentStringVerification getInstance() {
    return instance;
  }

  @Override
  public @NotNull ClassificationType getClassification() {
    return DecisionLogicLevelVerification.getInstance();
  }
}
